import { IKoffiLib, KoffiFunction, load } from "koffi";
import { Parser } from "../utils/getFuncsNamesFromVRHFile";
import path from "path";

export type RemoteFuncs = (typeof Remote.allowedFuncs)[number];

export class Remote {
  public static allowedFuncs = [
    "VBVMR_Login",
    "VBVMR_Logout",
    "VBVMR_GetVoicemeeterType",
    "VBVMR_SetParameterFloat",
    "VBVMR_SetParameterStringA",
    "VBVMR_SetParameters",
    "VBVMR_IsParametersDirty",
    "VBVMR_GetParameterFloat",
    "VBVMR_GetParameterStringA",
    "VBVMR_Output_GetDeviceNumber",
    "VBVMR_Input_GetDeviceNumber",
    "VBVMR_GetVoicemeeterVersion"
  ] as const;
  
  public dllFuncs = {} as Partial<
    Record<RemoteFuncs, KoffiFunction>
  >;
  public dll: IKoffiLib;

  constructor(dllPath: string) {
    this.dll = load(dllPath);
  }

  public async initialize() {
    const parser = new Parser(path.join(__dirname,"../", "VoicemeeterRemote.h"));
    const results = await parser.parse();
    for (const func of Remote.allowedFuncs) {
      const funcHeader = results.find((r) => r.name === func);
      if (funcHeader === undefined)
        throw new Error(`Allowed function not found in header file. Caused by function: ${func}`);
      this.dllFuncs[func] = this.dll.func(funcHeader.rawString);
    }
  }

  public run(
    command: RemoteFuncs,
    ...args: any
  ) {
    if (!Remote.allowedFuncs.includes(command))
      throw new Error("Used function isn't listed in allowed");
    const func = this.dllFuncs[command];
    if(func === undefined) throw new Error("Executed function wasn't initialized");
    return func(...args)
  }
}
